//============================================================================
// Name        : acat.cpp
// Author      : Martin Zak
//============================================================================

#include <iostream>
#include <string>
#include <set>
#include <cstdlib>
#include <unistd.h>
#include <exception>
#include "AutomatonFactory.h"
#include "GrammarFactory.h"
#include "AlibException.h"
#include "regexp/RegExpParser.h"
#include "automaton/AutomatonParser.h"
#include "grammar/GrammarParser.h"
#include "sax/SaxInterface.h"
#include "sax/ParserException.h"


#define VERSION "0.0.1"
#define USAGE "Usage: catPDA [-c] [input]";

using namespace std;
using namespace automaton;
using namespace regexp;
using namespace alib;
using namespace sax;
using namespace grammar;

void getAutomaton(list<Token>& tokens, bool complexTypes) {
	UnknownAutomaton automaton = AutomatonParser::parse(tokens);

	if(complexTypes) {
		Automaton* concreteAutomaton = AutomatonFactory::buildAutomaton(automaton);
		cout << *concreteAutomaton;
	} else {
		cout << automaton;
	}
}

void getGrammar(list<Token>& tokens) {
	UnknownGrammar grammar = GrammarParser::parse(tokens);
	cout << grammar;
}

void getRegExp(list<Token>& tokens) {
	RegExp regexp = RegExpParser::parse(tokens);
	cout << regexp;
}

int main(int argc, char** argv) {
	if(argc == 2 && string(argv[1])=="-v" ) {
			cout << argv[0] << " version " << VERSION << "\n";
			cout << USAGE;
			cout << endl;
			return EXIT_SUCCESS;
		}

	bool concrete = false;

	int i=getopt(argc, argv, "t:");
	while(i!=-1) {
		switch(i){
		case 'c':
			concrete = true;
			break;
		default:
			optind--;
			break;
		}
		i=getopt(argc, argv, "t:");
	}

	list<Token> tokens;
	if(optind == argc) {
		string input(istreambuf_iterator<char>(cin), (istreambuf_iterator<char>()));
		SaxInterface::parseMemory(input, tokens);
	} else {
		SaxInterface::parseFile(argv[optind],tokens);
	}

	if (tokens.front().getData() == "automaton") {
		getAutomaton(tokens, concrete);
	} else if (tokens.front().getData() == "grammar") {
		getGrammar(tokens);
	} else if (tokens.front().getData() == "regexp") {
		getRegExp(tokens);
	} else {
		throw AlibException( "Expected root tag automaton, grammar or regexp. Read: " + tokens.front().getData());
	}
}
