/*
 * Glushkov.h
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */

#ifndef GLUSHKOV_H_
#define GLUSHKOV_H_

#include <map>

#include <alphabet/Symbol.h>
#include <automaton/State.h>
#include <regexp/RegExp.h>
#include <regexp/RegExpElements.h>

#include "AbstractREtoFAConverter.h"
#include "../shared/GlushkovTraversal.h"

namespace conversions
{

/**
 * Converts regular expression to finite automata using Glushkov's NFA construction algorithm.
 * Source: Melichar 2.107
 */
class Glushkov : public AbstractREtoFAConverter
{
public:
    /**
     * @param re Original regular expression.
     */
    Glushkov( const regexp::RegExp & re );

    ~Glushkov( void );

    /**
     * Performs conversion.
     * @return FSM equivalent to original regular expression.
     *
     */
    automaton::FSM convert( void );

private:
    /**
     * Maps each symbol to unique number. Also creates automaton state.
     */
    struct NumberedSymbol
    {
        /**
         * Id of symbol (unique).
         */
        const int m_i;

        /**
         *
         */
        const alphabet::Symbol m_alphabetSymbol;

        /**
         * State which is created by this symbol in output automata.
         */
        const automaton::State m_state;

        /**
         * @param symb symbol
         * @param i unique number
         */
        NumberedSymbol( const regexp::RegExpSymbol * symb, int i );

        bool operator<( const NumberedSymbol & x ) const;
    };

    /**
     * Maps each symbol to unique number.
     */
    void initNumberSymbols( void );

    /**
     * Creates set of begin symbols
     * @see m_beginSymbolSet
     */
    void constructBeginSymbolSet( void );

    /**
     * Creates set of end symbols.
     * @see m_endSymbolSet
     */
    void constructEndSymbolSet( void );

    /**
     * Creates set of adjacent symbols.
     * @see m_neighbourSymbolSet
     */
    void constructNeighbourSymbolSet( void );

    std::map<const regexp::RegExpElement*, NumberedSymbol> m_numberedSymbols;
    std::set<NumberedSymbol> m_beginSymbolSet, m_endSymbolSet;
    std::set<GlushkovTraversal::Neighbours> m_neighbourSymbolSet;

};

} /* namespace conversions */

#endif /* GLUSHKOV_H_ */
