/*
 * BrzozowskiDerivationRRG.h
 *
 *  Created on: 6. 3. 2014
 *      Author: tomas
 */

#ifndef BRZOZOWSKIDERIVATIONRRG_H_
#define BRZOZOWSKIDERIVATIONRRG_H_

#include <deque>
#include <set>
#include <map>

#include <alphabet/Symbol.h>
#include <grammar/Regular/RightRegularGrammar.h>
#include <regexp/RegExp.h>

#include "AbstractREtoRRGConverter.h"

#include "RegExpOptimize.h"
#include "RegExpDerivation.h"

namespace conversions
{

/**
 * Converts reg. expression to right regular grammar using brzozowski derivation algorithm.
 * Source: Melichar 2.137
 */
class BrzozowskiDerivationRRG : public AbstractREtoRRGConverter
{
public:
    /**
     * @param re Source regular expression.
     */
    BrzozowskiDerivationRRG( const regexp::RegExp & re );

    ~BrzozowskiDerivationRRG( void );

    /**
     * Performs conversion.
     * @return right regular grammar equivalent to source regexp.
     */
    grammar::RightRegularGrammar convert( void );

protected:
    struct Transition
    {
        const regexp::RegExp m_from, m_to;
        const alphabet::Symbol m_symbol;

        Transition( const regexp::RegExp & from, const alphabet::Symbol & symb, const regexp::RegExp & to );
        bool operator<( const Transition & other ) const;
    };

    class NonTerminalBuilder
    {
    public:
        NonTerminalBuilder( const std::set<regexp::RegExp> & Q, grammar::RightRegularGrammar & grammar );
        const alphabet::Symbol & getNonTerminal( const regexp::RegExp & re ) const;

    private:
        grammar::RightRegularGrammar & m_grammar;
        std::map<const regexp::RegExp, alphabet::Symbol> m_nonTerminals;
        std::string createNewName( void );
        unsigned int m_nonTerminalId;
    };

    std::set<Transition> m_transitions;
};

} /* namespace conversions */

#endif /* BRZOZOWSKIDERIVATIONRRG_H_ */
