/*
 * GlushkovRRG.cpp
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */

#include "GlushkovRRG.h"

using namespace alib;
using namespace alphabet;
using namespace grammar;
using namespace regexp;
using namespace std;

namespace conversions
{

GlushkovRRG::GlushkovRRG( const RegExp & re ) : AbstractREtoRRGConverter( re )
{

}

GlushkovRRG::~GlushkovRRG( void )
{

}

RightRegularGrammar GlushkovRRG::convert( void )
{
    // step 1
    for( const auto & symbol : m_re.getAlphabet( ) )
        m_grammar.addTerminalSymbol( symbol.getSymbol( ) );

    initNumberSymbols( );

    // steps 2, 3, 4
    constructBeginSymbolSet( );
    constructEndSymbolSet( );
    constructNeighbourSymbolSet( );
        // \e in q0 check is in step 7

    // step 5
    Symbol S = m_grammar.createUniqueNonTerminalSymbol( "S" );
    m_grammar.setStartSymbol( S );

    //for( const auto & kv : m_numberedSymbols )
        //m_grammar.createUniqueNonTerminalSymbol( kv.second.m_nonTerminal );

    // step 6
    for( const auto & ns : m_beginSymbolSet )
    {
        list<Symbol> leftSide = { S };
        list<Symbol> rightSide = { ns.m_alphabetSymbol, ns.m_nonTerminal };
        m_grammar.addRule( Rule( leftSide, rightSide ) );
    }

    for( const auto & ns : m_neighbourSymbolSet )
    {
        const NumberedSymbol & first  = m_numberedSymbols.find( ns.m_first )->second;
        const NumberedSymbol & second = m_numberedSymbols.find( ns.m_second )->second;

        list<Symbol> leftSide = { first.m_nonTerminal };
        list<Symbol> rightSide = { second.m_alphabetSymbol, second.m_nonTerminal };

        m_grammar.addRule( Rule( leftSide, rightSide ) );
    }

    // step 7
    for( const auto & ns : m_endSymbolSet )
    {
        /*
         * for all rules where ns.m_nonTerminal is on rightSide:
         *  add Rule: leftSide -> ns.m_alphabetSymbol
         *  unless it already exists
         */

        for( const auto & rule : m_grammar.getRules( ) )
        {
            if( isInList( ns.m_nonTerminal, rule.getRightSide( ) ) )
            {
                list<Symbol> leftSide = rule.getLeftSide( );
                list<Symbol> rightSide = { ns.m_alphabetSymbol };
                Rule r( leftSide, rightSide );
                if( ! isInSet( r, m_grammar.getRules( ) ) )
                    m_grammar.addRule( r );
            }
        }
    }

    if( m_re.containsEmptyString( ) )
    {
        list<Symbol> leftSide = { S };
        list<Symbol> rightSide = { };
        m_grammar.addRule( Rule( leftSide, rightSide ) );
    }

    return m_grammar;
}

void GlushkovRRG::initNumberSymbols( void )
{
    int id = 1;

    for( const auto & symb : GlushkovTraversal::getSymbolsListInOrder( m_re ) )
        m_numberedSymbols.insert( pair<const RegExpSymbol*, NumberedSymbol>( symb, NumberedSymbol( symb, m_grammar, id ++ ) ) );
}

// ----------------------------------------------------------------------------

void GlushkovRRG::constructBeginSymbolSet( void )
{
    for( const auto & s : GlushkovTraversal::getLeftmostSymbolsInTree( m_re ) )
        m_beginSymbolSet.insert( m_numberedSymbols.find( s )->second );
}

void GlushkovRRG::constructEndSymbolSet( void )
{
    for( const auto & s : GlushkovTraversal::getRightmostSymbolsInTree( m_re ) )
        m_endSymbolSet.insert( m_numberedSymbols.find( s )->second );
}

void GlushkovRRG::constructNeighbourSymbolSet( void )
{
    for( const auto & n : GlushkovTraversal::getNeighbours( m_re ) )
        m_neighbourSymbolSet.insert( n );
}

// ----------------------------------------------------------------------------

bool GlushkovRRG::NumberedSymbol::operator<( const NumberedSymbol & x ) const
{
    return m_i < x.m_i;
}

GlushkovRRG::NumberedSymbol::NumberedSymbol( const RegExpSymbol * symbol, grammar::RightRegularGrammar & g, int i ) :
        m_i( i ),
        m_alphabetSymbol( symbol->getSymbol( ) ),
        // m_nonTerminal( g.createUniqueNonTerminalSymbol( "["     + m_alphabetSymbol.getSymbol( ) + "], id " + to_string( m_i ), false ) )
        m_nonTerminal( g.createUniqueNonTerminalSymbol( to_string( m_i ), false ) )
{

}

// ----------------------------------------------------------------------------



} /* namespace conversions */
