/*
 * GlushkovRRG.h
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */

#ifndef GLUSHKOVRRG_H_
#define GLUSHKOVRRG_H_

#include <map>

#include <grammar/Regular/RightRegularGrammar.h>
#include <regexp/RegExp.h>

#include "../../shared/GlushkovTraversal.h"
#include "AbstractREtoRRGConverter.h"

namespace conversions
{

/**
 * Converts regular expression to right regular grammar using Glushkov algorithm.
 * Source: None yet.
 */
class GlushkovRRG : public AbstractREtoRRGConverter
{
public:
    /**
     * @param re Source regular expresison.
     */
    GlushkovRRG( const regexp::RegExp & re );

    ~GlushkovRRG( void );

    /**
     * Performs conversion.
     * @return right regular grammar equivalent to source regexp.
     */
    grammar::RightRegularGrammar convert( void );

private:
    struct NumberedSymbol
    {
        const int m_i;
        const alphabet::Symbol m_alphabetSymbol;
        const alphabet::Symbol m_nonTerminal;

        NumberedSymbol( const regexp::RegExpSymbol * symb, grammar::RightRegularGrammar & rrg, int i );
        bool operator<( const NumberedSymbol & x ) const;
    };

    void initNumberSymbols( void );
    void constructBeginSymbolSet( void );
    void constructEndSymbolSet( void );
    void constructNeighbourSymbolSet( void );

    std::map<const regexp::RegExpElement*, NumberedSymbol> m_numberedSymbols;
    std::set<NumberedSymbol> m_beginSymbolSet, m_endSymbolSet;
    std::set<GlushkovTraversal::Neighbours> m_neighbourSymbolSet;
};

} /* namespace conversions */

#endif /* GLUSHKOVRRG_H_ */
