#include "RRGtoFAConverter.h"

using namespace alphabet;
using namespace automaton;
using namespace grammar;

namespace conversions
{

RRGtoFAConverter::RRGtoFAConverter( const RightRegularGrammar & grammar ) : AbstractRRGtoFAConverter( grammar )
{
    //TODO: if is not RRG throw Exception
}

RRGtoFAConverter::~RRGtoFAConverter( void )
{

}


FSM RRGtoFAConverter::convert( void )
{
    for( const auto & symbol : m_grammar.getTerminalSymbols( ) )
        m_automaton.addInputSymbol( symbol );

    for( const auto & symbol : m_grammar.getNonTerminalSymbols( ) )
        m_automaton.addState( State( symbol.getSymbol( ) ) );

    const State & AState = m_automaton.createUniqueState( "A", true );

    for( const auto & rule : m_grammar.getRules( ) )
    {
        if( m_grammar.isEpsilonRule( rule ) )
            continue;

        State current( rule.getLeftSide( ).front( ).getSymbol( ) );

        if( rule.getRightSide( ).size( ) == 1 ) // if B->a => \delta(B,a)=AState
        {
            const Symbol & input( rule.getRightSide( ).front( ).getSymbol( ) );
            m_automaton.addTransition( current, input, AState );
        }
        else if( rule.getRightSide( ).size( ) == 2 ) // if B->aC => \delta(B,a)=C
        {
            const Symbol & input = rule.getRightSide( ).front( );
            State next( rule.getRightSide( ).back( ).getSymbol( ) );

            m_automaton.addTransition( current, input, next );
        }
    }

    // step 4
    m_automaton.addInitialState( m_grammar.getStartSymbol().getSymbol() );

    // step 5
    m_automaton.addFinalState( AState );
    for( const auto & rule : m_grammar.getRules( ) )
        if( m_grammar.isEpsilonRule( rule ) )
            m_automaton.addFinalState( State( m_grammar.getStartSymbol( ).getSymbol( ) ) );

    return m_automaton;
}

} /* namespace conversions */
