/*
 * LRGAlgebraic.cpp
 *
 *  Created on: 4. 3. 2014
 *      Author: tomas
 */

#include "LRGAlgebraic.h"

using namespace alphabet;
using namespace grammar;
using namespace regexp;

namespace conversions
{

LRGAlgebraic::LRGAlgebraic( const LeftRegularGrammar & rg ) : AbstractLRGtoREConverter( rg )
{

}

LRGAlgebraic::~LRGAlgebraic( void )
{

}

RegExp LRGAlgebraic::convert( void )
{
    for( const auto & symb : m_grammar.getNonTerminalSymbols( ) )
        m_solver.addSymbol( symb );

    for( const auto & rule : m_grammar.getRules( ) )
    {
        const Symbol & leftSide = rule.getLeftSide( ).front( );
        if( rule.getRightSide( ).size( ) == 0 )
        {
            m_solver.addEquation( leftSide, new RegExpEpsilon( ) );
        }
        else if( rule.getRightSide( ).size( ) == 1 )
        {
            const Symbol & input = rule.getRightSide( ).front( );
            m_solver.addEquation( leftSide, new RegExpSymbol( input.getSymbol( ) ) );
        }
        else
        {
            const Symbol nt = rule.getRightSide( ).front( );
            const Symbol input = rule.getRightSide( ).back( );
            m_solver.addEquation( leftSide, nt, new RegExpSymbol( input.getSymbol( ) ) );
        }
    }

    return m_solver.solve( m_grammar.getStartSymbol( ) );
}

} /* namespace conversions */
