#include "RegExpParser.h"

RegExpParser::RegExpParser(RegExpLexer& lexer) : m_Lexer(lexer) {

}

regexp::RegExp* RegExpParser::parse() {
  regexp::RegExpElement* element = this->Alternation();
  RegExpLexer::Token token = m_Lexer.token();
  if(token.type != RegExpLexer::TEOF) throw ParseError();
  return new regexp::RegExp(element);
}

regexp::RegExpElement* RegExpParser::Alternation() {
  return this->AlternationCont(this->Concatenation());
}

regexp::RegExpElement* RegExpParser::AlternationCont(regexp::RegExpElement* left) {
  RegExpLexer::Token token = m_Lexer.token();
  if(token.type == RegExpLexer::PLUS) {
    m_Lexer.next();
    regexp::Alternation* res = this->AlternationContCont(this->Concatenation());
    res->getElements().push_front(left);
    return res;
  } else {
    return left;
  }
}

regexp::Alternation* RegExpParser::AlternationContCont(regexp::RegExpElement* left) {
  RegExpLexer::Token token = m_Lexer.token();
  if(token.type == RegExpLexer::PLUS) {
    m_Lexer.next();
    regexp::Alternation* res = this->AlternationContCont(this->Concatenation());
    res->getElements().push_front(left);
    return res;
  } else {
    regexp::Alternation* res = new regexp::Alternation();
    res->getElements().push_front(left);
    return res;
  }
}


regexp::RegExpElement* RegExpParser::Concatenation() {
  return this->ConcatenationCont(this->Factor());
}

regexp::RegExpElement* RegExpParser::ConcatenationCont(regexp::RegExpElement* left) {
  RegExpLexer::Token token = m_Lexer.token();
  if(token.type == RegExpLexer::SYMBOL || token.type == RegExpLexer::LPAR || token.type == RegExpLexer::EPS || token.type == RegExpLexer::EMPTY) {
    
    regexp::Concatenation* res = this->ConcatenationContCont(this->Factor());
    res->getElements().push_front(left);
    return res;
  } else {
    return left;
  }
}

regexp::Concatenation* RegExpParser::ConcatenationContCont(regexp::RegExpElement* left) {
  RegExpLexer::Token token = m_Lexer.token();
  if(token.type == RegExpLexer::SYMBOL || token.type == RegExpLexer::LPAR || token.type == RegExpLexer::EPS || token.type == RegExpLexer::EMPTY) {
    
    regexp::Concatenation* res = this->ConcatenationContCont(this->Factor());
    res->getElements().push_front(left);
    return res;
  } else {
    regexp::Concatenation* res = new regexp::Concatenation();
    res->getElements().push_front(left);
    return res;
  }
}

regexp::RegExpElement* RegExpParser::Factor() {
  RegExpLexer::Token token = m_Lexer.token();
  if(token.type == RegExpLexer::LPAR) {
    m_Lexer.next();
    regexp::RegExpElement* base = this->Alternation();
    token = m_Lexer.token();
    if(token.type != RegExpLexer::RPAR) throw ParseError();
    m_Lexer.next();
    return this->Star(base);
  } else if(token.type == RegExpLexer::EPS) {
    m_Lexer.next();
    return this->Star(new regexp::RegExpEpsilon());
  } else if(token.type == RegExpLexer::EMPTY) {
    m_Lexer.next();
    return this->Star(new regexp::RegExpEmpty());
  } else if(token.type == RegExpLexer::SYMBOL) {
    std::string symbol = token.value;
    m_Lexer.next();
    return this->Star(new regexp::RegExpSymbol(symbol));
  } else {
    throw ParseError();
  }
}

regexp::RegExpElement* RegExpParser::Star(regexp::RegExpElement* elem) {
  RegExpLexer::Token token = m_Lexer.token();
  if(token.type == RegExpLexer::STAR) {
    m_Lexer.next();
    regexp::Iteration* iter = new regexp::Iteration();
    iter->setElement(this->Star(elem));
    return iter;
  } else {
    return elem;
  }
}
