#ifndef VPASTRUCTS_H_
#define VPASTRUCTS_H_

#include <set>

using namespace automaton;

namespace determinization {
namespace vpa {


/**
 * Struct fot storing pair of two states from vpa.
 */
struct StatesPair
{
    State state1;
    State state2;
    
    StatesPair(const State& state1, const State& state2)
            : state1(state1),
              state2(state2) {}
    
    bool operator < (const StatesPair& other) const
    {
        return (state1 < other.state1) || ((state1 == other.state1) && (state2 < other.state2));
    }
    
    bool operator == (const StatesPair& other) const
    {
        return (state1 == other.state1) && (state2 == other.state2);
    }
    
    bool operator != (const StatesPair& other) const
    {
        return (state1 != other.state1) || (state2 != other.state2);
    }
};


/**
 * Struct for storing S component of state from deterministic vpa. S component consists of set of states pairs.
 */
struct SComponent
{
    set<StatesPair> statesPairs;
    SComponent() {}
    SComponent(const set<StatesPair>& statesPairs)
            : statesPairs(statesPairs) {}
};


/**
 * Struct for storing R component of state from deterministic vpa. R component consists of set of states.
 */
struct RComponent
{
    set<State> states;
    RComponent() {}
    RComponent(const set<State>& states)
            : states(states) {}
};


/**
 * Struct for storing data about state from deterministic vpa.
 */
struct StateData
{

    /** State of deterministic vpa */
    State state;

    /** S component of this state */
    SComponent s;

    /** R component of this state */
    RComponent r;

    /** Indicates if this state is already marked by determinization algorithm or not */
    bool isMarked;
    
    StateData(const State& state, const SComponent& s, const RComponent& r)
            : state(state),
              s(s),
              r(r),
              isMarked(false) {}
    
    StateData(const State& state, const SComponent& s)
            : state(state),
              s(s),
              isMarked(false) {}

};


}
}
#endif
