/*
 * TransitionTM.h
 *
 *  Created on: Apr 24, 2013
 *      Author: martin
 */

#ifndef TRANSITIONTM_H_
#define TRANSITIONTM_H_

#include "../Transition.h"
#include "../Shift.h"

namespace automaton {

using namespace std;

class TransitionTM: public Transition {
protected:
	Symbol output;
	Shift shift;
public:

	/**
	 * Creates new transition with given parameters.
	 * @param from from state
	 * @param input input symbol
	 * @param to to state
	 * @param output output symbol
	 * @param shift direction of movement of the reading head
	 */
	TransitionTM(const State& from, const Symbol& input, const State& to, const Symbol& output, Shift shift);

	/**
	 * @return output Symbol
	 */
	const Symbol& getOutput() const;

	/**
	 * Sets the output symbol.
	 * @param output Symbol
	 */
	void setOutput(const Symbol& output);

	/**
	 * @return direction of movement of the reading head
	 */
	Shift getShift() const;

	/**
	 * Sets the direction of movement of the reading head
	 * @param shift Shift to set
	 */
	void setShift(Shift shift);

	bool operator <(const TransitionTM& other) const;
	bool operator ==(const TransitionTM& other) const;
	bool operator !=(const TransitionTM& other) const;

	std::ostream& operator>>(std::ostream& out) const;
};

} /* namespace automaton */
#endif /* TRANSITIONTM_H_ */
