/*
 * RegExp.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#include "RegExp.h"
#include "RegExpPrinter.h"

#include <iostream>

namespace regexp {

RegExp::RegExp() {
	regExp = NULL;
}

RegExp::RegExp(const RegExp& other) {
	if (other.regExp != NULL) {
		regExp = other.regExp->clone();
	} else {
		regExp = NULL;
	}
}

RegExp::RegExp(const RegExpElement* regExp) {
	if (regExp != NULL) {
		this->regExp = regExp->clone();
	} else {
		this->regExp = NULL;
	}
}

RegExp& RegExp::operator =(const RegExp& other) {
	if (this == &other) {
		return *this;
	}

	if (regExp != NULL) {
		delete regExp;
	}

	if (other.regExp != NULL) {
		regExp = other.regExp->clone();
	} else {
		regExp = NULL;
	}

	return *this;
}

RegExp::~RegExp() {
	if (regExp != NULL) {
		delete regExp;
	}

	regExp = NULL;
}

RegExpElement* RegExp::getRegExp() {
	return regExp;
}

const RegExpElement* RegExp::getRegExp() const {
	return regExp;
}

void RegExp::setRegExp(RegExpElement* regExp) {
	this->regExp = regExp;
}

set<alphabet::Symbol> RegExp::getAlphabet(void) const{
	set<alphabet::Symbol> alphabet;

	if(regExp)
		regExp->getAlphabet( alphabet );

	return alphabet;
}

bool RegExp::isEmpty() const {
	return regExp == NULL || regExp->isEmpty();
}

bool RegExp::containsEmptyString() const {
	if(regExp)
		return regExp->containsEmptyString();

	return false;
}

void RegExp::toXML(ostream& out) const {
	RegExpPrinter::toXML(*this, out);
}

ostream& operator <<(ostream& out, const RegExp& regexp) {
	regexp.toXML(out);
	return out;
}

bool RegExp::operator<(const RegExp& other) const {
	return *(this->regExp) < *(other.regExp);
}

bool RegExp::operator<=(const RegExp& other) const {
	return *(this->regExp) <= *(other.regExp);
}

bool RegExp::operator==(const RegExp& other) const {
	return *(this->regExp) == *(other.regExp);
}

bool RegExp::operator!=(const RegExp& other) const {
	return *(this->regExp) != *(other.regExp);
}

bool RegExp::operator>(const RegExp& other) const {
	return *(this->regExp) > *(other.regExp);
}

bool RegExp::operator>=(const RegExp& other) const {
	return *(this->regExp) >= *(other.regExp);
}

} /* namespace regexp */
