/*
 * RegExpPrinter.h
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#ifndef REGEXPPRINTER_H_
#define REGEXPPRINTER_H_

#include <ostream>
#include "RegExp.h"
#include "RegExpElements.h"

namespace regexp {

using namespace std;

/**
 * This class contains methods to print XML representation of regular expression to the output stream.
 */
class RegExpPrinter {
protected:
	static const string INDENTATION;
	static void printElement(const RegExpElement* element, ostream& out, const string & prefix);
	static void printContent(const list<RegExpElement*>& content, ostream& out, const string & prefix);
	static void printAlternation(const Alternation* alternation, ostream& out, const string & prefix);
	static void printConcatenation(const Concatenation* concatenation, ostream& out, const string & prefix);

	static void printIteration(const Iteration* iteration, ostream& out, const string & prefix);
	static void printSymbol(const RegExpSymbol* symbol, ostream& out, const string & prefix);
	static void printEpsilon(const RegExpEpsilon* symbol, ostream& out, const string & prefix);
	static void printEmpty(const RegExpEmpty* symbol, ostream& out, const string & prefix);

public:
	/**
	 * Prints XML representation of RegExp to the output stream.
	 * @param regexp RegExp to print
	 * @param out output stream to which print the RegExp
	 */
	static void toXML(const RegExp& regexp, ostream& out);
};

} /* namespace regexp */
#endif /* REGEXPPRINTER_H_ */
