/*
 * Normalize.cpp
 *
 *  Created on: Dec 9, 2013
 *      Author: honza
 */

#include "Normalize.h"

#include <map>
#include <deque>
#include <vector>
#include <set>
#include <algorithm>
#include <sstream>
#include <iostream>
#include "AlibException.h"

#include "alphabet/Symbol.h"
#include "automaton/FSM/TransitionFSM.h"

automaton::State fromInteger(int number) {
  std::stringstream ss;
  ss << number;
  return automaton::State(ss.str());
}

bool transitionSymbolComparator(const automaton::TransitionFSM& a, const automaton::TransitionFSM& b) {
  return a.getInput() < b.getInput();
}

automaton::FSM Normalize::normalize(automaton::FSM& fsm) {
	int counter = 0;
	std::map<automaton::State, int > normalizationData;
	std::deque<automaton::State > processingData;
	
	normalizationData.insert(std::pair<automaton::State, int>(*(fsm.getInitialStates().begin()), counter++));
	processingData.push_back(*(fsm.getInitialStates().begin()));
	
	while(!processingData.empty()) {
		automaton::State current = processingData.front();
		processingData.pop_front();
		
		const std::set<automaton::TransitionFSM> transitionsSet = fsm.getTransitionsFromState(current);
		std::vector<automaton::TransitionFSM> transitions;
		
		std::copy(transitionsSet.begin(), transitionsSet.end(), std::back_inserter(transitions));
		std::sort (transitions.begin(), transitions.end(), transitionSymbolComparator);
		
		for(auto iter = transitions.begin(); iter != transitions.end(); iter++) {
			if(normalizationData.find(iter->getTo()) == normalizationData.end()) {
				normalizationData.insert(std::pair<automaton::State, int>(iter->getTo(), counter++));
				processingData.push_back(iter->getTo());
			}
		}
	}
	
	if(normalizationData.size() != fsm.getStates().size()) {
		throw alib::AlibException("Automaton normalize require minimal deterministic finite automaton");
	}
	
	automaton::FSM result;
	for(auto iter = fsm.getInputAlphabet().begin(); iter != fsm.getInputAlphabet().end(); iter++) {
		result.addInputSymbol(*iter);
	}
	
	for(auto iter = fsm.getStates().begin(); iter != fsm.getStates().end(); iter++) {
		result.addState(fromInteger(normalizationData.find(*iter)->second));
	}
	
	for(auto iter = fsm.getInitialStates().begin(); iter != fsm.getInitialStates().end(); iter++) {
		result.addInitialState(fromInteger(normalizationData.find(*iter)->second));
	}
	
	for(auto iter = fsm.getFinalStates().begin(); iter != fsm.getFinalStates().end(); iter++) {
		result.addFinalState(fromInteger(normalizationData.find(*iter)->second));
	}
	
	for(auto iter = fsm.getTransitions().begin(); iter != fsm.getTransitions().end(); iter++) {
		result.addTransition(fromInteger(normalizationData.find(iter->getFrom())->second), iter->getInput(), fromInteger(normalizationData.find(iter->getTo())->second));
	}

	return result;
}
